package org.sc3d.apt.sss.v3;

/** An extensible buffer for Tokens. */
public class TokenBuffer {
  /** Constructs an empty TokenBuffer. */
  public TokenBuffer() {
    this.buffer = new Token[10];
    this.used = 0;
  }
  
  /* New API. */
  
  /** Appends 't' to this TokenBuffer.
   * @return 'this', for convenience.
   */
  public TokenBuffer append(Token t) {
    if (this.used==this.buffer.length) {
      final Token[] old = this.buffer;
      this.buffer = new Token[2*old.length];
      System.arraycopy(old, 0, this.buffer, 0, this.used);
    }
    this.buffer[this.used++] = t;
    return this;
  }
  
  /** Returns the number of Tokens in this TokenBuffer. */
  public int length() { return this.used; }
  
  /** Returns the Token at index 'index' in this TokenBuffer. */
  public Token get(int index) {
    if (index>this.used) throw new ArrayIndexOutOfBoundsException();
    return this.buffer[index];
  }
  
  /** Returns the contents of this TokenBuffer in a fresh array of length 'length()'. */
  public Token[] toArray() {
    final Token[] ans = new Token[this.used];
    System.arraycopy(this.buffer, 0, ans, 0, this.used);
    return ans;
  }
  
  /* Override things in Object. */
  
  /** Returns a String representation of this TokenBuffer including its contents. */
  public String toString() {
    final StringBuffer ans = new StringBuffer("TokenBuffer[");
    for (int i=0; i<this.used; i++) ans.append("\n  ").append(this.buffer[i]);
    return ans.append("\n]").toString();
  }
  
  /* Private. */
  
  /** Storage for the Tokens in this TokenBuffer. */
  private Token[] buffer;
  
  /** The number of Tokens in this TokenBuffer. */
  private int used;
}
